@@ -31,13 +31,13 @@
 #include "mapserver.h"
 #include "mapcopy.h"
 #include "mapresample.h"
-#ifdef USE_GDAL
 
 #include "ogr_api.h"
 #include "ogr_srs_api.h"
 #include "gdal.h"
 #include "gdal_alg.h"
 
+#include "mapows.h"
 #include "mapthread.h"
 #include "mapraster.h"
 #include "cpl_string.h"
@@ -112,6 +112,21 @@ static void msContourLayerInfoInitialize(layerObj *layer)
   clinfo->ogrLayer.connection = (char*)msSmallMalloc(strlen(clinfo->ogrLayer.name)+13);
   sprintf(clinfo->ogrLayer.connection, "__%s_CONTOUR__", clinfo->ogrLayer.name);
   clinfo->ogrLayer.units = layer->units;
+
+  if (msOWSLookupMetadata(&(layer->metadata), "OFG", "ID_type") == NULL) {
+    msInsertHashTable(&(layer->metadata), "gml_ID_type", "Integer");
+  }
+  {
+    const char* elevItem = CSLFetchNameValue(layer->processing,"CONTOUR_ITEM");
+    if (elevItem && strlen(elevItem) > 0) {
+       char szTmp[100];
+       snprintf(szTmp, sizeof(szTmp), "%s_type", elevItem);
+       if (msOWSLookupMetadata(&(layer->metadata), "OFG", szTmp) == NULL) {
+         snprintf(szTmp, sizeof(szTmp), "gml_%s_type", elevItem);
+         msInsertHashTable(&(layer->metadata), szTmp, "Real");
+       }
+    }
+  }
 }
 
 static void msContourLayerInfoFree(layerObj *layer)
@@ -233,8 +248,11 @@ static int msContourLayerReadRaster(layerObj *layer, rectObj rect)
     InvGeoTransform(adfGeoTransform, adfInvGeoTransform);
 
     mapRect = rect;
-    map_cellsize_x = map_cellsize_y = map->cellsize;      
-#ifdef USE_PROJ
+    if( map->cellsize == 0 )
+    {
+        map->cellsize = msAdjustExtent(&mapRect,map->width,map->height);
+    }
+    map_cellsize_x = map_cellsize_y = map->cellsize;
     /* if necessary, project the searchrect to source coords */
     if (msProjectionsDiffer( &(map->projection), &(layer->projection)))  {
       if ( msProjectRect(&map->projection, &layer->projection, &mapRect)
@@ -267,8 +285,7 @@ static int msContourLayerReadRaster(layerObj *layer, rectObj rect)
                                          MS_CELLSIZE(rect.miny, rect.maxy, map->height));
       }       
     }
-#endif
-    
+
     if (map_cellsize_x == 0 || map_cellsize_y == 0) {
       if (layer->debug)
         msDebug("msContourLayerReadRaster(): Cellsize can't be 0.\n");
@@ -776,6 +793,7 @@ int msContourLayerClose(layerObj *layer)
 
 int msContourLayerGetItems(layerObj *layer)
 {
+  const char* elevItem;
   contourLayerInfo *clinfo = (contourLayerInfo *) layer->layerinfo;
 
   if (clinfo == NULL) {
@@ -784,7 +802,16 @@ int msContourLayerGetItems(layerObj *layer)
     return MS_FAILURE;
   }
 
-  return msContourLayerGetItems(&clinfo->ogrLayer);
+  layer->numitems = 0;
+  layer->items = (char **) msSmallCalloc(sizeof(char *),2);
+
+  layer->items[layer->numitems++] = msStrdup("ID");
+  elevItem = CSLFetchNameValue(layer->processing,"CONTOUR_ITEM");
+  if (elevItem && strlen(elevItem) > 0) {
+    layer->items[layer->numitems++] = msStrdup(elevItem);
+  }
+
+  return msLayerGetItems(&clinfo->ogrLayer);
 }
 
 int msContourLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
@@ -802,9 +829,13 @@ int msContourLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
     return MS_FAILURE;
   }
 
-  newRect = rect;
-  
-#ifdef USE_PROJ
+  if( isQuery )
+  {
+    newRect = layer->map->extent;
+  }
+  else
+  {
+    newRect = rect;
     /* if necessary, project the searchrect to source coords */
     if (msProjectionsDiffer( &(layer->map->projection), &(layer->projection)))  {
       if (msProjectRect(&layer->projection, &layer->map->projection, &newRect)
@@ -813,7 +844,7 @@ int msContourLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
         return MS_FAILURE;
       }
     }
-#endif
+  }
 
   /* regenerate the raster io */
   if (clinfo->hOGRDS)
@@ -991,12 +1022,3 @@ int msContourLayerInitializeVirtualTable(layerObj *layer)
 
   return MS_SUCCESS;
 }
-
-#else
-int msContourLayerInitializeVirtualTable(layerObj *layer)
-{
-  msSetError(MS_MISCERR, "Contour Layer needs GDAL support, but it it not compiled in", "msContourLayerInitializeVirtualTable()");
-  return MS_FAILURE;
-}
-#endif
-
